

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "mpegaudioheader.h"


const int BitrateLookup[16][6] =
{
	{0,  0,  0,  0,  0,  0},
	{32, 32, 32, 32, 32, 8 /* 8 */},
	{64, 48, 40, 64, 48, 16 /* 16 */},
	{96, 56, 48, 96, 56, 24, /* 24 */},
	{128, 64, 56, 128, 64, 32 /* 32 */},
	{160, 80, 64, 160, 80, 64 /* 40 */},
	{192, 96, 80, 192, 96, 80 /* 48 */},
	{224, 112, 96, 224, 112, 56 /* 56 */},
	{256, 128, 112, 256, 128, 64 /* 64 */},
	{288, 160, 128, 288, 160, 128 /* 80 */},
	{320, 192, 160, 320, 192, 160 /* 96 */},
	{352, 224, 192, 352, 224, 112 /* 112 */},
	{384, 256, 224, 384, 256, 128 /* 128 */},
	{416, 320, 256, 416, 320, 256 /* 144 */},
	{448, 384, 320, 448, 384, 320 /* 160 */},
	{-1, -1, -1, -1, -1, -1,}
};


int mpeg_audio_header_find ( FILE * f, mpeg_audio_header_t * pHeader )
{
	unsigned long ulBytesOfHeaderRead = 0;
	
	while (ulBytesOfHeaderRead < MPEG_AUDIO_HEADER_SIZE)
	{
		if (fread(&pHeader->headerData[ulBytesOfHeaderRead],1,1,f)==1)
		{
			switch (ulBytesOfHeaderRead)
			{
			
			case 0:
				if (pHeader->headerData[0]==0xff)
				{
					ulBytesOfHeaderRead++;				
				}
				break;
				
			case 1:
				if ((pHeader->headerData[1] & 0xe0)==0xe0)
				{				
					pHeader->MPEGAudioVersion = (pHeader->headerData[1] & 0x18) >> 3;
					pHeader->layerDescription = (pHeader->headerData[1] & 0x06) >> 1;
					pHeader->protectionBit = (pHeader->headerData[1] & 0x01);
				
					ulBytesOfHeaderRead++;
				}
				else
				{
					ulBytesOfHeaderRead = 0;
				}
				break;
				
			case 2:
			
				pHeader->bitrateIndex = (pHeader->headerData[2] & 0xf0) >> 4;
				pHeader->samplingRateFrequencyIndex = (pHeader->headerData[2] & 0x0c) >> 2;
				pHeader->paddingBit = (pHeader->headerData[2] & 0x02) >> 1;
				pHeader->privateBit = (pHeader->headerData[2] & 0x01);

				ulBytesOfHeaderRead++;
			
				break;
				
			case 3:
			
				pHeader->channelMode = (pHeader->headerData[3] & 0xc0) >> 6;
				pHeader->modeExtension = (pHeader->headerData[3] & 0x30) >> 4;
				pHeader->copyright = (pHeader->headerData[3] & 0x08) >> 3;
				pHeader->original = (pHeader->headerData[3] & 0x04) >> 2;
				pHeader->emphasis = pHeader->headerData[3] & 0x03;

				ulBytesOfHeaderRead++;
			
				break;
				
			}
		}
		else
		{
			return 0;
		}
	}
	
	return 1;
}

int mpeg_audio_header_layer_index(mpeg_audio_header_t * pHeader)
{
	switch (pHeader->layerDescription)
	{
	case MPEG_AUDIO_LAYER_RESERVED:
		return 0;
	case MPEG_AUDIO_LAYER_III:
		return 3;
	case MPEG_AUDIO_LAYER_II:
		return 2;
	case MPEG_AUDIO_LAYER_I:
		return 1;

	}
	return 0;
}



int mpeg_audio_header_bitrate(mpeg_audio_header_t * pHeader)
{
	int iLayerIndex = mpeg_audio_header_layer_index(pHeader);
	
	
	/* printf ("layer index %d bitrate index %d.\n", iLayerIndex, pHeader->bitrateIndex); */
	
	if ((iLayerIndex>=1) && (iLayerIndex<=3))
	{
		if (pHeader->bitrateIndex<=15)
		{
			if (pHeader->MPEGAudioVersion==MPEG_AUDIO_VERSION_2 || 
				pHeader->MPEGAudioVersion==MPEG_AUDIO_VERSION_2_5)
			{
				return BitrateLookup[pHeader->bitrateIndex][3+(iLayerIndex-1)];
			}
			else if (pHeader->MPEGAudioVersion==MPEG_AUDIO_VERSION_1)
			{
				return BitrateLookup[pHeader->bitrateIndex][iLayerIndex-1];
			}
		}
	}
	return -1;
}

int mpeg_audio_header_sampling_rate(mpeg_audio_header_t * pHeader)
{

	switch (pHeader->MPEGAudioVersion)
	{
	case MPEG_AUDIO_VERSION_1:
		switch (pHeader->samplingRateFrequencyIndex)
		{
		case 0x0: return 44100;
		case 0x1: return 48000;
		case 0x2: return 32000;
		}
		break;

	case MPEG_AUDIO_VERSION_2:
		switch (pHeader->samplingRateFrequencyIndex)
		{
		case 0x0: return 22050;
		case 0x1: return 24000;
		case 0x2: return 16000;
		}
		break;

	case MPEG_AUDIO_VERSION_2_5:
		switch (pHeader->samplingRateFrequencyIndex)
		{
		case 0x0: return 11025;
		case 0x1: return 12000;
		case 0x2: return 8000;
		}
		break;		
	}
	return -1;
}


int mpeg_audio_header_frame_size(mpeg_audio_header_t * pHeader)
{
	int iBitrate = mpeg_audio_header_bitrate(pHeader);
	int iSamplingRate = mpeg_audio_header_sampling_rate(pHeader);
	
	/* printf ("bitrate %d sample rate %d.\n", iBitrate, iSamplingRate); */

	if (iBitrate>0 && iSamplingRate>0)
	{
		return 144 * (iBitrate * 1000) / iSamplingRate + pHeader->paddingBit;
	}
	else
	{
		return 0;
	}
}

long long mpeg_audio_header_duration(mpeg_audio_header_t * pHeader, long long clock_rate)
{
	long long llFrameSizeBits = mpeg_audio_header_frame_size(pHeader) * 8;
	
	if (llFrameSizeBits)
	{
		long long llBitrate = (long long) mpeg_audio_header_bitrate(pHeader);

		if (llBitrate > 0)
		{
			return (llFrameSizeBits*clock_rate) / (llBitrate * clock_rate);
		}
	}

	return 0;
}

